! module with the integrator (now Runge-Kutta-Gauss, 3 stages)
module integr

  implicit none

  integer, parameter :: S=3 ! stage of RK Gauss method
  double precision :: eta(S)
  double precision :: beta(S)
  double precision :: rho(S,S)

contains

!********************************************************************************************************

  ! Runge-Kutta-Gauss integrator
  ! 
  subroutine rkg(X, t, force, Dt, Tspan, printFunc)

     abstract interface
        ! checkpoint function
        subroutine checkpoint(XX, tt)
           double precision,intent(in):: XX(:), tt
        end subroutine checkpoint
     end interface
     double precision,intent(inOut):: X(:), t
     interface
        function force(XX,tt)
           double precision,intent(in):: XX(:), tt
           double precision:: force(size(XX))
        end function force
     end interface
     double precision,intent(in):: Dt
     double precision,intent(in):: Tspan
     procedure(checkpoint):: printFunc

     integer,parameter:: ITMAX = 20 
     double precision,parameter :: tol= 1.d-16
     integer:: i, j, k, jj, it
     integer :: numit, dimx
     double precision :: xold(size(X)),xnew(size(X)),xi(size(X))
     double precision :: told,tnew,ti
     double precision :: fVal(size(X))
     double precision :: Txit(size(X),S)
     double precision :: ep(size(X),S)
     double precision :: de

     ! Tableau RK-Gauss dimension S=3 (change it if you need higher dimension)
     eta = (/0.1127016653792583D+00, 0.5000000000000000D+00, 0.8872983346207417D+00/)
     beta = (/0.2777777777777778D+00, 0.4444444444444444D+00, 0.2777777777777778D+00/)
     rho = reshape((/  0.1388888888888889D+00,  0.3002631949808646D+00, 0.2679883337624695D+00, &
                      -0.3597666752493890D-01,  0.2222222222222222D+00, 0.4804211119693833D+00, &
          0.9789444015308326D-02, -0.2248541720308681D-01, 0.1388888888888889D+00/), shape(rho))

     dimx=size(X)

     xold=X
     told=t

     numit = INT(Tspan/Dt)+1 

     ! propagation
     do k=1,numit
         
        write(*,fmt="(A1,A,t21,F6.2,A)",advance="NO") ACHAR(13),           &
           & " Percent Complete: " ,                                       &
             ( real(k) / real(numit))*100.0, "%"  

        ! initialization
        Txit(1:dimx,1) = force(xold,told)
        do j = 2,S
           Txit(1:dimx,j) = Txit(1:dimx,1)
        endDo
        ep = 0.d0
        ep(1,1) = 1.d0 

        it = 1

        ! implicit scheme
        do while ((MAXVAL(ep).gt.tol).and.(it.lt.ITMAX))
           ep=0.d0
           do j = 1,S
              do i = 1,dimx
                 de = 0.d0
                 do jj = 1,S
                    de = de + rho(j,jj)*Txit(i,jj)
                 endDo
                 xi(i) = de*Dt + xold(i)
              endDo
              ti = told + eta(j)*Dt
              fVal = force(xi,ti)
              do i = 1,dimx
                 if(ABS(fVal(i)).ne.0.d0)then
                    ep(:,j) = ABS(fVal(i) - Txit(i,j))/ABS(fVal(i))
                 endif
              endDo
              Txit(:,j) = fVal
           endDo
           it = it + 1
        endDo

        ! check for bad convergence
        !if (it.eq.ITMAX) then
        !   write(*,*) 'number of iteration reached the maximum', it
        !endif

        ! rkg step
        do i = 1,dimx
           de = 0.d0
           do j = 1,S
              de = de + beta(j)*Txit(i,j)
          endDo
          xnew(i) = de*Dt + xold(i)
        endDo
        tnew = told + Dt

        ! output
        call printFunc(xnew,tnew)

        ! save for the next step
        xold = xnew
        told = tnew

     endDo

     X=xnew
     t=tnew

  end subroutine rkg

!***************************************************************************************************

end module integr
