! module for handling input files and define hamiltonian and integration parameters
module parameters

  implicit none

  !-------------------------------------------------------------------------
  ! Parameters required for computing the Hamiltonian function
  ! 
  type hamparam

     double precision:: GM                     ! gravitational parameter
     double precision:: alpha                  ! inertia constant
     double precision:: n                      ! mean motion
     double precision:: dotw                   ! variation of spin rate due to dissipation

  end type hamparam
!-------------------------------------------------------------------------
  ! Parameters of the numerical integration
  ! 
  type intparam

     double precision:: t0    ! initial time
     double precision:: tSpan ! duration
     double precision:: Dt    ! time step
     integer:: Nsave          ! number of timesteps between each save

  end type intparam

!-------------------------------------------------------------------------
contains
!***************************************************************************************

  ! Read the hamiltonian parameters from an input file and rewrite them in an output file.
  ! All the parameters should have the same system of units (e.g. au, years, Sun-mass).
  ! 
  subroutine readHamParam(ficIn,ficOut,Hp)
     character(len=*),intent(in):: ficIn, ficOut
     type(hamparam),intent(out):: Hp

     double precision:: GM        
     double precision:: alpha  
     double precision:: dotw
     integer:: Nreg      

     namelist /physicalparams/ GM, alpha, dotw

     !-----------------------------------------------------------------------
     ! Reading the parameters
     open(newunit=Nreg,file=trim(ficIn),status="OLD",action="READ")
     read(Nreg,physicalparams)
     close(Nreg)

     ! Writing the parameters
     open(newunit=Nreg,file=trim(ficOut),status="REPLACE",action="WRITE")

     write(Nreg,'(A)')              "&physicalparams"
     write(Nreg,'(A,ES22.15,A)')    "    GM = " , GM,","
     write(Nreg,'(A,ES22.15,A)')    " alpha = " , alpha,","
     write(Nreg,'(A,ES22.15,A)')    "  dotw = " , dotw,","

     close(Nreg)
     !-----------------------------------------------------------------------

     Hp%GM = GM
     Hp%alpha = alpha
     Hp%dotw = dotw

  end subroutine readHamParam

!***************************************************************************************

  ! Read the parameters of the numerical integration and the initial conditions in an
  ! input file, and rewrite them in an output file.
  ! 
  subroutine initInteg(ficIn,ficOut, Hp,Ip, Z0)
     character(len=*),intent(in):: ficIn, ficOut
     type(hamparam),intent(inout):: Hp
     type(intparam),intent(out):: Ip
     double precision,intent(out):: Z0(6)


     double precision:: t0    ! initial time
     double precision:: tSpan ! duration
     double precision:: Dt    ! time step
     integer:: Nsave          ! number of timesteps between each save

     double precision:: X(4)
     double precision:: E,n,T
     integer:: Nreg,i   

     namelist /integrationparam/ t0, tSpan, Dt, Nsave, X

     !-----------------------------------------------------------------------
     ! Reading the parameters
     open(newunit=Nreg,file=trim(ficIn),status="OLD",action="READ")
     read(Nreg,integrationparam)
     close(Nreg)

     ! Writing the parameters
     open(newunit=Nreg,file=trim(ficOut),status="REPLACE",action="WRITE")

     write(Nreg,'(A)')              "&integrationparam"
     write(Nreg,'(A,ES22.15,A)')    "     t0 = " , t0,","
     write(Nreg,'(A,ES22.15,A)')    "  tSpan = " , tSpan,","
     write(Nreg,'(A,ES22.15,A)')    "     Dt = " , Dt,","
     write(Nreg,'(A,I0,A)')         "  Nsave =  ", Nsave,","
     write(Nreg,'(A,*(ES22.15,A))') "      X = " , (X(i),",",i=1,4)
     write(Nreg,'(A)') "/"

     close(Nreg)


     ! initial conditions
     ! Z0=(theta, dot theta, x, y, vx, vy)
     ! initial position taken at the pericenter
     
     Z0(1)=X(1)
     Z0(3)=X(3)*(1-X(4))  ! x=a*(1-e)
     Z0(4)=0.d0           ! y=0
     Z0(5)=0.d0           ! vx=0
     
     E=-Hp%GM/(2*X(3))              ! energy per unit mass
     n=sqrt(Hp%GM/X(3)**3)          ! mean motion
     T=2.d0*3.141592653589793238462643D0/n
     Z0(2)=X(2)*n                   ! omega
     Z0(6)=sqrt(2*(E+Hp%GM/Z0(3)))  ! vy

     !-----------------------------------------------------------------------
     ! save mean motion value in Hamiltonian parameters
     Hp%n=n

     !-----------------------------------------------------------------------

     ! Parameters of the numerical integration
     ! time parameters taken as multiples of orbital period
     Ip%t0      = t0
     Ip%tSpan   = tSpan*T
     Ip%Dt      = Dt*T
     Ip%Nsave   = Nsave

     !-----------------------------------------------------------------------

  end subroutine initInteg

!***************************************************************************************
end module parameters
